@@ -1,18 +1,18 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2011 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2012 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
-# Python X2go is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
+# Python X2Go is free software; you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as published by
 # the Free Software Foundation; either version 3 of the License, or
 # (at your option) any later version.
 #
-# Python X2go is distributed in the hope that it will be useful,
+# Python X2Go is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# GNU Affero General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
+# You should have received a copy of the GNU Affero General Public License
 # along with this program; if not, write to the
 # Free Software Foundation, Inc.,
 # 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
@@ -20,7 +20,7 @@
 """\
 L{X2goSessionProfiles} class - managing x2goclient session profiles.
 
-L{X2goSessionProfiles} is a public API class. Use this class in your Python X2go based 
+L{X2goSessionProfiles} is a public API class. Use this class in your Python X2Go based 
 applications.
 
 """
@@ -29,7 +29,7 @@ __NAME__ = 'x2gosessionprofiles-pylib'
 import copy
 import types
 
-# Python X2go modules
+# Python X2Go modules
 from x2go.defaults import X2GO_SESSIONPROFILES_CONFIGFILES as _X2GO_SESSIONPROFILES_CONFIGFILES
 from x2go.defaults import X2GO_SESSIONPROFILE_DEFAULTS as _X2GO_SESSIONPROFILE_DEFAULTS
 from x2go.defaults import X2GO_DESKTOPSESSIONS as _X2GO_DESKTOPSESSIONS
@@ -44,10 +44,22 @@ class X2goSessionProfilesFILE(inifiles.X2goIniFile):
     defaultSessionProfile = _X2GO_SESSIONPROFILE_DEFAULTS
     _non_profile_sections = ('embedded')
 
-
     def __init__(self, config_files=_X2GO_SESSIONPROFILES_CONFIGFILES, defaults=None, session_profile_defaults=None, logger=None, loglevel=log.loglevel_DEFAULT):
         """\
-        STILL UNDOCUMENTED
+        Retrieve X2Go session profiles from a file, typically C{~/.x2goclient/sessions}.
+
+        @param config_files: a list of config file locations, the first file name in this list the user has write access to will be the user configuration file
+        @type config_files: C{list}
+        @param defaults: not used for this class
+        @type defaults: C{dict}
+        @param session_profile_defaults: a default session profile
+        @type session_profile_defaults: C{dict}
+        @param logger: you can pass an L{X2goLogger} object to the
+            L{X2goSessionProfilesFILE} constructor
+        @type logger: L{X2goLogger} instance
+        @param loglevel: if no L{X2goLogger} object has been supplied a new one will be
+            constructed with the given loglevel
+        @type loglevel: C{int}
 
         """
         self.defaultValues = {}
@@ -70,20 +82,38 @@ class X2goSessionProfilesFILE(inifiles.X2goIniFile):
 
         self.session_profiles = [ p for p in self.iniConfig.sections() if p not in self._non_profile_sections ]
         for session_profile in self.session_profiles:
-            self.get_profile_metatype(session_profile)
             for key, default_value in self.defaultSessionProfile.iteritems():
                 if not self.iniConfig.has_option(session_profile, key):
                     self._storeValue(session_profile, key, default_value)
+            self.get_profile_metatype(session_profile)
 
     def __call__(self, profile_id_or_name):
         """\
-        STILL UNDOCUMENTED
+        Retrieve the session profile configuration for a given session profile ID (or name)
+
+        @param profile_id_or_name: profile ID or profile name
+        @type profile_id_or_name: C{str}
+
+        @return: the profile ID's / name's profile configuration
+        @rtype: C{dict}
 
         """
-        _profile_id = check_profile_id_or_name(self, profile_id_or_name)
+        _profile_id = self.check_profile_id_or_name(self, profile_id_or_name)
         return self.get_profile_config(profile_id=_profile_id)
 
     def get_profile_metatype(self, profile_id_or_name, force=False):
+        """\
+        Detect a human readable session profile type from the session profile configuration.
+
+        @param profile_id_or_name: profile ID or profile name
+        @type profile_id_or_name: C{str}
+        @param force: re-detect the meta type, otherwise use a cached result
+        @type force: C{bool}
+
+        @return: the profile ID's / name's meta type
+        @rtype: C{str}
+
+        """
 
         _profile_id = self.check_profile_id_or_name(profile_id_or_name)
         if not self._profile_metatypes.has_key(_profile_id) or force:
@@ -91,15 +121,21 @@ class X2goSessionProfilesFILE(inifiles.X2goIniFile):
             if _config['host']:
                 if _config['rdpserver'] and _config['command'] == 'RDP':
                     _metatype = 'RDP/proxy'
+                elif _config['published']:
+
+                    if _config['command'] in _X2GO_DESKTOPSESSIONS.keys():
+                        _metatype = '%s + Published Applications' % _config['command']
+                    else:
+                        _metatype = 'Published Applications'
+
                 elif _config['rootless']:
                     _metatype = 'Single Applications'
+                elif _config['command'] in _X2GO_DESKTOPSESSIONS.keys():
+                    _metatype = '%s Desktop' % _config['command']
+                elif _config['command'] in _X2GO_DESKTOPSESSIONS.values():
+                    _metatype = '%s Desktop' % [ s for s in _X2GO_DESKTOPSESSIONS.keys() if _config['command'] == _X2GO_DESKTOPSESSIONS[s] ][0]
                 else:
-                    if _config['command'] in _X2GO_DESKTOPSESSIONS.keys():
-                        _metatype = '%s Desktop' % _config['command']
-                    elif _config['command'] in _X2GO_DESKTOPSESSIONS.values():
-                        _metatype = '%s Desktop' % [ s for s in _X2GO_DESKTOPSESSIONS.keys() if _config['command'] == _X2GO_DESKTOPSESSIONS[s] ][0]
-                    else:
-                        _metatype = 'CUSTOM Desktop'
+                    _metatype = 'CUSTOM Desktop'
             else:
                 if _config['rdpserver'] and _config['command'] == 'RDP':
                     _metatype = 'RDP/direct'
@@ -111,7 +147,13 @@ class X2goSessionProfilesFILE(inifiles.X2goIniFile):
 
     def get_profile_option_type(self, option):
         """\
-        STILL UNDOCUMENTED
+        Get the data type for a specific session profile option.
+
+        @param option: the option to get the data type for
+        @type option: will be detected by this method
+
+        @return: the data type of C{option}
+        @rtype: C{type}
 
         """
         try:
@@ -121,7 +163,15 @@ class X2goSessionProfilesFILE(inifiles.X2goIniFile):
 
     def get_type(self, section, key):
         """\
-        STILL UNDOCUMENTED
+        Override the parent class's get_type method due to the special layout of this class.
+
+        @param section: INI file section
+        @type section: C{str}
+        @param key: key in INI file section
+        @type key: C{str}
+
+        @return: the data type of C{key} in C{section}
+        @rtype: C{type}
 
         """
         # we have to handle the get_type method separately...
@@ -129,7 +179,15 @@ class X2goSessionProfilesFILE(inifiles.X2goIniFile):
 
     def get_profile_config(self, profile_id_or_name=None, profile_id=None):
         """\
-        STILL UNDOCUMENTED
+        The configuration options for a single session profile.
+
+        @param profile_id_or_name: either profile ID or profile name is accepted
+        @type profile_id_or_name: C{str}
+        @param profile_id: profile ID (fast than specifying C{profile_id_or_name})
+        @type profile_id: C{str}
+
+        @return: the session profile configuration for the given profile ID (or name)
+        @rtype: C{dict}
 
         """
         _profile_id = profile_id or self.check_profile_id_or_name(profile_id_or_name)
@@ -140,14 +198,27 @@ class X2goSessionProfilesFILE(inifiles.X2goIniFile):
 
     def default_profile_config(self):
         """\
-        STILL UNDOCUMENTED
+        Return a default session profile.
+
+        @return: default session profile
+        @rtype: C{dict}
 
         """
         return copy.deepcopy(self.defaultSessionProfile)
 
     def has_profile(self, profile_id_or_name):
+        """\
+        Does a session profile of a given profile ID or profile name exist?
+
+        @param profile_id_or_name: profile ID or profile name
+        @type profile_id_or_name: C{str}
+
+        @return: C{True} if there is such a session profile, C{False} otherwise
+        @rtype: C{bool}
+
+        """
         try:
-            _profile_id = self.check_profile_id_or_name(profile_id_or_name)
+            self.check_profile_id_or_name(profile_id_or_name)
             return True
         except X2goProfileException:
             return False
@@ -155,7 +226,7 @@ class X2goSessionProfilesFILE(inifiles.X2goIniFile):
     @property
     def profile_ids(self):
         """\
-        STILL UNDOCUMENTED
+        Renders a list of all profile IDs found in the session profile configuration file.
 
         """
         if not self._cached_profile_ids:
@@ -164,7 +235,13 @@ class X2goSessionProfilesFILE(inifiles.X2goIniFile):
 
     def has_profile_id(self, profile_id):
         """\
-        STILL UNDOCUMENTED
+        Does a session profile of a given profile ID exist? (Faster than L{has_profile()}.)
+
+        @param profile_id: profile ID
+        @type profile_id: C{str}
+
+        @return: C{True} if there is such a session profile, C{False} otherwise
+        @rtype: C{bool}
 
         """
         return profile_id in self.profile_ids
@@ -172,7 +249,7 @@ class X2goSessionProfilesFILE(inifiles.X2goIniFile):
     @property
     def profile_names(self):
         """\
-        STILL UNDOCUMENTED
+        Renders a list of all profile names found in the session profile configuration file.
 
         """
         if not self._cached_profile_names:
@@ -181,14 +258,26 @@ class X2goSessionProfilesFILE(inifiles.X2goIniFile):
 
     def has_profile_name(self, profile_name):
         """\
-        STILL UNDOCUMENTED
+        Does a session profile of a given profile name exist? (Faster than L{has_profile()}.)
+
+        @param profile_name: profile name
+        @type profile_name: C{str}
+
+        @return: C{True} if there is such a session profile, C{False} otherwise
+        @rtype: C{bool}
 
         """
         return profile_name in self.profile_names
 
     def to_profile_id(self, profile_name):
         """\
-        STILL UNDOCUMENTED
+        Convert profile name to profile ID.
+
+        @param profile_name: profile name
+        @type profile_name: C{str}
+
+        @return: profile ID
+        @rtype: C{str}
 
         """
         _profile_ids = [ p for p in self.profile_ids if self.to_profile_name(p) == profile_name ]
@@ -201,7 +290,13 @@ class X2goSessionProfilesFILE(inifiles.X2goIniFile):
 
     def to_profile_name(self, profile_id):
         """\
-        STILL UNDOCUMENTED
+        Convert profile ID to profile name.
+
+        @param profile_id: profile ID
+        @type profile_id: C{str}
+
+        @return: profile name
+        @rtype: C{str}
 
         """
         _profile_config = self.get_profile_config(profile_id=profile_id)
@@ -212,7 +307,15 @@ class X2goSessionProfilesFILE(inifiles.X2goIniFile):
 
     def add_profile(self, profile_id=None, **kwargs):
         """\
-        STILL UNDOCUMENTED
+        Add a new session profile.
+
+        @param profile_id: a custom profile ID--if left empty a profile ID will be auto-generated
+        @type profile_id: C{str}
+        @param kwargs: session profile options for this new session profile
+        @type kwargs: C{dict}
+
+        @return: the (auto-generated) profile ID of the new session profile
+        @rtype: C{str}
 
         """
         if profile_id is None:
@@ -221,7 +324,7 @@ class X2goSessionProfilesFILE(inifiles.X2goIniFile):
             if key in self.defaultSessionProfile:
                 self.update_value(profile_id, key, value)
             else:
-                raise X2goProfileException('keyword ,,%s\'\' not supported in X2go session profile' % key)
+                raise X2goProfileException('keyword ,,%s\'\' not supported in X2Go session profile' % key)
 
         for key, value in self.defaultSessionProfile.items():
             if key in kwargs: continue
@@ -234,7 +337,10 @@ class X2goSessionProfilesFILE(inifiles.X2goIniFile):
 
     def delete_profile(self, profile_id_or_name):
         """\
-        STILL UNDOCUMENTED
+        Delete a session profile from the configuration file.
+
+        @param profile_id_or_name: profile ID or profile name
+        @type profile_id_or_name: C{str}
 
         """
         _profile_id = self.check_profile_id_or_name(profile_id_or_name)
@@ -246,7 +352,14 @@ class X2goSessionProfilesFILE(inifiles.X2goIniFile):
 
     def update_value(self, section, key, value):
         """\
-        STILL UNDOCUMENTED
+        Update a value in a session profile.
+
+        @param section: the profile ID
+        @type section: C{str}
+        @param key: the session profile option of the given profile ID
+        @type key: C{str}
+        @param value: the value to update the session profile option with
+        @type value: any type, depends on the session profile option
 
         """
         profile_id = section
@@ -263,7 +376,15 @@ class X2goSessionProfilesFILE(inifiles.X2goIniFile):
 
     def check_profile_id_or_name(self, profile_id_or_name):
         """\
-        STILL UNDOCUMENTED
+        Detect the profile ID from a given string which maybe profile ID or profile name.
+
+        @param profile_id_or_name: profile ID or profile name
+        @type profile_id_or_name: C{str}
+
+        @return: profile ID
+        @rtype: C{str}
+
+        @raise X2goProfileException: if no such session profile exists
 
         """
         _profile_id = None
@@ -279,7 +400,15 @@ class X2goSessionProfilesFILE(inifiles.X2goIniFile):
 
     def to_session_params(self, profile_id_or_name=None, profile_id=None):
         """\
-        STILL UNDOCUMENTED
+        Convert session profile options to L{X2goSession} constructor method parameters.
+
+        @param profile_id_or_name: either profile ID or profile name is accepted
+        @type profile_id_or_name: C{str}
+        @param profile_id: profile ID (fast than specifying C{profile_id_or_name})
+        @type profile_id: C{str}
+
+        @return: a dictionary of L{X2goSession} constructor method parameters
+        @rtype: C{dict}
 
         """
         _profile_id = profile_id or self.check_profile_id_or_name(profile_id_or_name)
@@ -287,7 +416,15 @@ class X2goSessionProfilesFILE(inifiles.X2goIniFile):
 
     def get_session_param(self, profile_id_or_name, param):
         """\
-        STILL UNDOCUMENTED
+        Get a single L{X2goSession} parameter from a specific session profile.
+
+        @param profile_id_or_name: either profile ID or profile name is accepted
+        @type profile_id_or_name: C{str}
+        @param param: the parameter name in the L{X2goSession} constructor method
+        @type param: C{str}
+
+        @return: the value of the session profile option represented by C{param}
+        @rtype: depends on the session profile option requested
 
         """
         return self.to_session_params(profile_id_or_name)[param]
