@@ -1,18 +1,18 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2011 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2012 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
-# Python X2go is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
+# Python X2Go is free software; you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as published by
 # the Free Software Foundation; either version 3 of the License, or
 # (at your option) any later version.
 #
-# Python X2go is distributed in the hope that it will be useful,
+# Python X2Go is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# GNU Affero General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
+# You should have received a copy of the GNU Affero General Public License
 # along with this program; if not, write to the
 # Free Software Foundation, Inc.,
 # 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
@@ -26,14 +26,11 @@ __NAME__ = 'x2goproxy-pylib'
 # modules
 import gevent
 import os
-import sys
-import types
-import time
 import copy
 import threading
-import cStringIO
+import socket
 
-# Python X2go modules
+# Python X2Go modules
 import x2go.forward as forward
 import x2go.log as log
 import x2go.utils as utils
@@ -52,7 +49,7 @@ from x2go.defaults import X2GO_SESSIONS_ROOTDIR as _X2GO_SESSIONS_ROOTDIR
 
 class X2goProxyBASE(threading.Thread):
     """\
-    X2goProxy is an abstract class for X2go proxy connections.
+    X2goProxy is an abstract class for X2Go proxy connections.
 
     This class needs to be inherited from a concrete proxy class. Only 
     currently available proxy class is: L{X2goProxyNX3}.
@@ -85,7 +82,7 @@ class X2goProxyBASE(threading.Thread):
         @type ssh_transport: C{paramiko.Transport} instance
         @param session_log: name of the proxy's session logfile
         @type session_log: C{str}
-        @param sessions_rootdir: base dir where X2go session files are stored (by default: ~/.x2go)
+        @param sessions_rootdir: base dir where X2Go session files are stored (by default: ~/.x2go)
         @type sessions_rootdir: C{str}
         @param proxy_options: a set of very C{X2goProxy*} backend specific options; any option that is not known
             to the C{X2goProxy*} backend will simply be ignored
@@ -120,23 +117,23 @@ class X2goProxyBASE(threading.Thread):
 
     def __del__(self):
         """\
-        On instance destruction make sure the this proxy thread is stopped properly.
+        On instance destruction make sure this proxy thread is stopped properly.
 
         """
         self.stop_thread()
 
     def _tidy_up(self):
         """\
-        Close any left open port forwarding tunnel, also close session log file, 
+        Close any left open port forwarding tunnel, also close session log file,
         if left open.
 
         """
         if self.proxy:
-            self.logger('Shutting down X2go proxy subprocess', loglevel=log.loglevel_DEBUG)
+            self.logger('Shutting down X2Go proxy subprocess', loglevel=log.loglevel_DEBUG)
             try:
                 self.proxy.kill()
             except OSError, e:
-                self.logger('X2go proxy shutdown gave a message that we may ignore: %s' % str(e), loglevel=log.loglevel_WARN)
+                self.logger('X2Go proxy shutdown gave a message that we may ignore: %s' % str(e), loglevel=log.loglevel_WARN)
             self.proxy = None
         if self.fw_tunnel is not None:
             self.logger('Shutting down Paramiko/SSH forwarding tunnel', loglevel=log.loglevel_DEBUG)
@@ -159,7 +156,7 @@ class X2goProxyBASE(threading.Thread):
 
     def run(self):
         """\
-        Start the X2go proxy command. The X2go proxy command utilizes a
+        Start the X2Go proxy command. The X2Go proxy command utilizes a
         Paramiko/SSH based forwarding tunnel (openssh -L option). This tunnel
         gets started here and is forked into background (Greenlet/gevent).
 
@@ -171,17 +168,23 @@ class X2goProxyBASE(threading.Thread):
             return None
 
         try:
-            os.mkdir(self.session_info.local_container)
+            os.makedirs(self.session_info.local_container)
         except OSError, e:
             if e.errno == 17:
                 # file exists
                 pass
 
         local_graphics_port = self.session_info.graphics_port
-        local_graphics_port = utils.detect_unused_port(bind_address='', preferred_port=local_graphics_port)
+        try:
+            if self.ssh_transport.getpeername() in ('::1', '127.0.0.1', 'localhost', 'localhost.localdomain'):
+                local_graphics_port += 10000
+        except socket.error:
+            raise x2go_exceptions.X2goControlSessionException('The control session has died unexpectedly.')
+        local_graphics_port = utils.detect_unused_port(preferred_port=local_graphics_port)
+
         self.fw_tunnel = forward.start_forward_tunnel(local_port=local_graphics_port, 
-                                                      remote_port=self.session_info.graphics_port, 
-                                                      ssh_transport=self.ssh_transport, 
+                                                      remote_port=self.session_info.graphics_port,
+                                                      ssh_transport=self.ssh_transport,
                                                       session_instance=self.session_instance,
                                                       logger=self.logger,
                                                      )
@@ -257,7 +260,7 @@ class X2goProxyBASE(threading.Thread):
         Start the thread runner and wait for the proxy to come up.
 
         @return: a subprocess instance that knows about the externally started proxy command.
-        @rtype: C{instance}
+        @rtype: C{obj}
 
         """
         threading.Thread.start(self)
@@ -270,22 +273,24 @@ class X2goProxyBASE(threading.Thread):
             self.logger('waiting for proxy to come up: 0.4s x %s' % _count, loglevel=log.loglevel_DEBUG)
             gevent.sleep(.4)
 
-        # also wait for fw_tunnel to become active
-        _count = 0
-        _maxwait = 40
-        while not self.fw_tunnel.is_active and _count < _maxwait:
-            _count += 1
-            self.logger('waiting for port fw tunnel to come up: 0.5s x %s' % _count, loglevel=log.loglevel_DEBUG)
-            gevent.sleep(.5)
+        if self.proxy:
 
-        return self.proxy
+            # also wait for fw_tunnel to become active
+            _count = 0
+            _maxwait = 40
+            while (not self.fw_tunnel.is_active) and (not self.fw_tunnel.failed) and (_count < _maxwait):
+                _count += 1
+                self.logger('waiting for port fw tunnel to come up: 0.5s x %s' % _count, loglevel=log.loglevel_DEBUG)
+                gevent.sleep(.5)
+
+        return self.proxy, bool(self.proxy) and self.fw_tunnel.is_active
 
     def ok(self):
         """\
         Check if a proxy instance is up and running.
 
-        @return: Proxy state (C{True} or C{False})
+        @return: Proxy state, C{True} for proxy being up-and-running, C{False} otherwise
         @rtype C{bool}
 
         """
-        return bool(self.proxy and self.proxy.poll() is None)
+        return bool(self.proxy and self.proxy.poll() is None) and self.fw_tunnel.is_active
