@@ -1,24 +1,24 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2011 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2012 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
-# Python X2go is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
+# Python X2Go is free software; you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as published by
 # the Free Software Foundation; either version 3 of the License, or
 # (at your option) any later version.
 #
-# Python X2go is distributed in the hope that it will be useful,
+# Python X2Go is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# GNU Affero General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
+# You should have received a copy of the GNU Affero General Public License
 # along with this program; if not, write to the
 # Free Software Foundation, Inc.,
 # 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 
 """\
-A recommended X2go session clean up helper function.
+A recommended X2Go session clean up helper function.
 
 """
 
@@ -26,7 +26,7 @@ import gevent
 import paramiko
 import threading
 
-# Python X2go modules
+# Python X2Go modules
 import guardian
 import rforward
 from defaults import X2GOCLIENT_OS as _X2GOCLIENT_OS
@@ -37,7 +37,7 @@ if _X2GOCLIENT_OS == 'Windows':
 
 def x2go_cleanup(e=None, threads=None):
     """\
-    For every Python X2go application you write, please make sure to 
+    For every Python X2Go application you write, please make sure to 
     capture the C{KeyboardInterrupt} and the C{SystemExit} exceptions and 
     call this function if either of the exceptions occurs.
 
@@ -68,13 +68,13 @@ def x2go_cleanup(e=None, threads=None):
         else:
             threads = threads
 
-        # stop X2go reverse forwarding tunnels
+        # stop X2Go reverse forwarding tunnels
         for t in threads:
             if type(t) == rforward.X2goRevFwTunnel:
                 t.stop_thread()
                 del t
 
-        # stop X2go paramiko transports used by X2goTerminalSession objects
+        # stop X2Go paramiko transports used by X2goTerminalSession objects
         for t in threads:
             if type(t) == paramiko.Transport:
                 if hasattr(t, '_x2go_session_marker'):
@@ -86,7 +86,6 @@ def x2go_cleanup(e=None, threads=None):
             for t in threads:
                 if type(t) == xserver.X2goXServer:
                     t.stop_thread()
-                    gevent.sleep(1)
                     del t
 
         # on Windows: stop the PulseAudio daemon that we evoked
@@ -94,14 +93,20 @@ def x2go_cleanup(e=None, threads=None):
             for t in threads:
                 if type(t) == pulseaudio.X2goPulseAudio:
                     t.stop_thread()
-                    gevent.sleep(1)
                     del t
 
+        for t in threads:
+            if type(t) == guardian.X2goSessionGuardian:
+                t.stop_thread()
+                del t
+
+        gevent.sleep(1)
+
         if e is not None:
             raise e
 
     except KeyboardInterrupt:
-        # do not allow keyboard interrupts during Python X2go cleanup
+        # do not allow keyboard interrupts during Python X2Go cleanup
         pass
     except SystemExit:
         # neither do we allow SIGTERM signals during cleanup...
