@@ -1,43 +1,51 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2011 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2012 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
-# Python X2go is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
+# Python X2Go is free software; you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as published by
 # the Free Software Foundation; either version 3 of the License, or
 # (at your option) any later version.
 #
-# Python X2go is distributed in the hope that it will be useful,
+# Python X2Go is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# GNU Affero General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
+# You should have received a copy of the GNU Affero General Public License
 # along with this program; if not, write to the
 # Free Software Foundation, Inc.,
 # 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 
 """\
-Default variables and values for Python X2go.
+Default variables and values for Python X2Go.
 
 """
 __NAME__ = 'x2godefaults-pylib'
 
-import sys
 import os
 import paramiko
 import platform
 
-## X2go imports
-import utils
-
 ##
-## Common X2go defaults
+## Common X2Go defaults
 ##
 
 X2GOCLIENT_OS = platform.system()
 
-LOCAL_HOME = os.path.expanduser('~')
+if X2GOCLIENT_OS != 'Windows':
+    import Xlib.display
+    import Xlib.error
+
+    # handle missing X displays on package build
+    try:
+        X_DISPLAY = Xlib.display.Display()
+    except Xlib.error.DisplayNameError:
+	X_DISPLAY = None
+    except Xlib.error.DisplayConnectionError:
+        X_DISPLAY = None
+
+LOCAL_HOME = os.path.normpath(os.path.expanduser('~'))
 X2GO_SESSIONS_ROOTDIR = '.x2go'
 X2GO_CLIENT_ROOTDIR = '.x2goclient'
 X2GO_SSH_ROOTDIR = os.path.join('.x2go','.ssh')
@@ -119,49 +127,49 @@ BACKENDS_SESSIONPROFILES = {
     'HTTPSBROKER': 'X2goSessionProfilesHTTPSBROKER',
     'WINREG': 'X2goSessionProfilesWINREG',
 }
-"""Python X2go backends for storing session profiles."""
+"""Python X2Go backends for storing session profiles."""
 BACKENDS_CLIENTSETTINGS = {
     'FILE': 'X2goClientSettingsFILE',
     'GCONF': 'X2goClientSettingsGCONF',
     'HTTPSBROKER': 'X2goClientSettingsHTTPSBROKER',
     'WINREG': 'X2goClientSettingsWINREG',
 }
-"""Python X2go backends for storing client settings."""
+"""Python X2Go backends for storing client settings."""
 BACKENDS_CLIENTPRINTING = {
     'FILE': 'X2goClientPrintingFILE',
     'GCONF': 'X2goClientPrintingGCONF',
     'HTTPSBROKER': 'X2goClientPrintingHTTPSBROKER',
     'WINREG': 'X2goClientPrintingWINREG',
 }
-"""Python X2go backends for storing print settings."""
+"""Python X2Go backends for storing print settings."""
 
 BACKEND_SESSIONPROFILES_DEFAULT = 'X2goSessionProfilesFILE'
 BACKEND_CLIENTSETTINGS_DEFAULT = 'X2goClientSettingsFILE'
 BACKEND_CLIENTPRINTING_DEFAULT = 'X2goClientPrintingFILE'
 
 ##
-## X2go Printing
+## X2Go Printing
 ##
 
 X2GO_SETTINGS_FILENAME = 'settings'
 X2GO_SETTINGS_CONFIGFILES = [
-    os.path.join(LOCAL_HOME, X2GO_CLIENT_ROOTDIR, 'settings'),
-    os.path.join(ETC_DIR,X2GO_SETTINGS_FILENAME),
+    os.path.normpath(os.path.join(LOCAL_HOME, X2GO_CLIENT_ROOTDIR, 'settings')),
+    os.path.normpath(os.path.join(ETC_DIR,X2GO_SETTINGS_FILENAME)),
 ]
 X2GO_PRINTING_FILENAME = 'printing'
 X2GO_PRINTING_CONFIGFILES = [
-    os.path.join(LOCAL_HOME, X2GO_CLIENT_ROOTDIR, 'printing'),
-    os.path.join(ETC_DIR,X2GO_PRINTING_FILENAME),
+    os.path.normpath(os.path.join(LOCAL_HOME, X2GO_CLIENT_ROOTDIR, 'printing')),
+    os.path.normpath(os.path.join(ETC_DIR,X2GO_PRINTING_FILENAME)),
 ]
 X2GO_SESSIONPROFILES_FILENAME = 'sessions'
 X2GO_SESSIONPROFILES_CONFIGFILES = [
-    os.path.join(LOCAL_HOME, X2GO_CLIENT_ROOTDIR, 'sessions'),
-    os.path.join(ETC_DIR,X2GO_SESSIONPROFILES_FILENAME),
+    os.path.normpath(os.path.join(LOCAL_HOME, X2GO_CLIENT_ROOTDIR, 'sessions')),
+    os.path.normpath(os.path.join(ETC_DIR,X2GO_SESSIONPROFILES_FILENAME)),
 ]
 X2GO_XCONFIG_FILENAME = 'xconfig'
 X2GO_XCONFIG_CONFIGFILES = [
-    os.path.join(LOCAL_HOME, X2GO_CLIENT_ROOTDIR, 'xconfig'),
-    os.path.join(ETC_DIR,X2GO_XCONFIG_FILENAME),
+    os.path.normpath(os.path.join(LOCAL_HOME, X2GO_CLIENT_ROOTDIR, 'xconfig')),
+    os.path.normpath(os.path.join(ETC_DIR,X2GO_XCONFIG_FILENAME)),
 ]
 
 X2GO_CLIENTSETTINGS_DEFAULTS = {
@@ -233,10 +241,11 @@ if X2GOCLIENT_OS == 'Windows':
 if X2GOCLIENT_OS == 'Windows':
     X2GO_CLIENTXCONFIG_DEFAULTS = {
         'XServers': {
-            'known_xservers': ['VcXsrv', 'Xming', 'Cygwin-X', ],
+            'known_xservers': ['VcXsrv_development', 'VcXsrv_shipped', 'VcXsrv', 'Xming', 'Cygwin-X', ],
         },
         'Cygwin-X': {
             'display': 'localhost:40',
+            'last_display': 'localhost:40',
             'process_name': 'XWin.exe',
             'test_installed': os.path.join(os.environ['SystemDrive'], '\\', 'cygwin', 'bin', 'XWin.exe'),
             'run_command': os.path.join(os.environ['SystemDrive'], '\\', 'cygwin', 'bin', 'XWin.exe'),
@@ -244,13 +253,31 @@ if X2GOCLIENT_OS == 'Windows':
             },
         'VcXsrv': {
             'display': 'localhost:40',
+            'last_display': 'localhost:40',
             'process_name': 'vcxsrv.exe',
             'test_installed': os.path.join(os.environ['ProgramFiles'], 'VcXsrv', 'vcxsrv.exe'), 
             'run_command': os.path.join(os.environ['ProgramFiles'], 'VcXsrv', 'vcxsrv.exe'),
             'parameters': [':40', '-clipboard', '-multiwindow', '-notrayicon', '-nowinkill', '-nounixkill', '-swcursor', ],
             },
+        'VcXsrv_shipped': {
+            'display': 'localhost:40',
+            'last_display': 'localhost:40',
+            'process_name': 'vcxsrv.exe',
+            'test_installed': os.path.join(os.getcwd(), 'VcXsrv', 'vcxsrv.exe'), 
+            'run_command': os.path.join(os.getcwd(), 'VcXsrv', 'vcxsrv.exe'),
+            'parameters': [':40', '-clipboard', '-multiwindow', '-notrayicon', '-nowinkill', '-nounixkill', '-swcursor', ],
+            },
+        'VcXsrv_development': {
+            'display': 'localhost:40',
+            'last_display': 'localhost:40',
+            'process_name': 'vcxsrv.exe',
+            'test_installed': os.path.join(os.getcwd(), '..', 'pyhoca-contrib', 'mswin', 'vcxsrv-mswin', 'VcXsrv-1.12.0.1', 'vcxsrv.exe'), 
+            'run_command': os.path.join(os.getcwd(), '..', 'pyhoca-contrib', 'mswin', 'vcxsrv-mswin', 'VcXsrv-1.12.0.1', 'vcxsrv.exe'), 
+            'parameters': [':40', '-clipboard', '-multiwindow', '-notrayicon', '-nowinkill', '-nounixkill', '-swcursor', ],
+            },
         'Xming': {
             'display': 'localhost:40',
+            'last_display': 'localhost:40',
             'process_name': 'Xming.exe',
             'test_installed': os.path.join(os.environ['ProgramFiles'], 'Xming', 'Xming.exe'), 
             'run_command': os.path.join(os.environ['ProgramFiles'], 'Xming', 'Xming.exe'),
@@ -265,25 +292,26 @@ X2GO_GENERIC_APPLICATIONS = [ 'WWWBROWSER', 'MAILCLIENT', 'OFFICE', 'TERMINAL',
 """X2go's generic applications."""
 
 X2GO_SESSIONPROFILE_DEFAULTS = {
+    'autologin': False, 'autostart': False, 'setsessiontitle': False, 'sessiontitle': "",
     'speed': 2, 'pack': '16m-jpeg', 'quality': 9,
     'iconvto': 'UTF-8', 'iconvfrom': 'UTF-8', 'useiconv': False,
     'usesshproxy': False, 'sshproxyhost': '', 'sshproxyuser': '', 'sshproxytunnel': '', 'sshproxykeyfile': '',
     'useexports': True, 'fstunnel': True, 'export': '',
     'usemimebox': False, 'mimeboxextensions': '', 'mimeboxaction': 'OPEN',
     'fullscreen': False,
-    'width': 800,'height': 600,'dpi': 96,'setdpi': False,
-    'usekbd':True, 'layout': 'us', 'type': 'pc105/us',
-    'sound':False, 'soundsystem': 'pulse', 'startsoundsystem': False, 'soundtunnel':True, 'defsndport':True, 'sndport':4713, 
-    'name': 'NEW_PROFILE', 'icon': ':icons/128x128/x2gosession.png', 
-    'host': '', 'user': CURRENT_LOCAL_USER, 'key': '', 'sshport': 22,
-    'rootless': True, 'applications': X2GO_GENERIC_APPLICATIONS, 'command':'TERMINAL',
+    'width': 800,'height': 600,'dpi': 96,'setdpi': False, 'xinerama': False, 'multidisp': False,
+    'usekbd': True, 'layout': 'us', 'type': 'pc105/us', 'variant': '',
+    'sound': False, 'soundsystem': 'pulse', 'startsoundsystem': False, 'soundtunnel':True, 'defsndport':True, 'sndport':4713,
+    'name': 'NEW_PROFILE', 'icon': ':icons/128x128/x2gosession.png',
+    'host': '', 'user': CURRENT_LOCAL_USER, 'key': '', 'sshport': 22, 'krblogin': False,
+    'rootless': True, 'applications': X2GO_GENERIC_APPLICATIONS, 'command':'TERMINAL', 'published': False,
     'rdpoptions': '-u X2GO_USER -p X2GO_PASSWORD', 'rdpserver': '',
     'print': False,
     'xdmcpserver': 'localhost',
     }
 """L{X2goSessionProfiles} default values to fill a new session profile with."""
 ##
-## X2go Proxy defaults
+## X2Go Proxy defaults
 ##
 
 # here is a list of NX 3.x compression methods, this is the "%"-hashed list that
@@ -340,26 +368,27 @@ for meth in [ m for m in pack_methods_nx3_noqual if "%" in m ]:
     pack_methods_nx3 += [ meth.replace('%','%s' % str(i)) for i in range(0,10) ]
 pack_methods_nx3.sort()
 ##
-## X2go session defaults
+## X2Go session defaults
 ##
 
 X2GO_DESKTOPSESSIONS={
     'KDE': 'startkde',
     'GNOME': 'gnome-session',
+    'XFCE4': 'xfce4-session',
     'LXDE': 'startlxde',
     'TRINITY': 'starttrinity',
-    'UNITY': 'unity-2d-launcher',
+    'UNITY': 'unity',
 }
 """A dictionary with meta-commands for X2go's window manager sessions."""
 
 ##
-## X2go SFTP server defaults
+## X2Go SFTP server defaults
 ##
 
 RSAKEY_STRENGTH = 1024
 RSAHostKey = paramiko.RSAKey.generate(RSAKEY_STRENGTH)
 """\
-An RSA host key for this client session. Python X2go does not use the
+An RSA host key for this client session. Python X2Go does not use the
 system's host key but generates its own host key for each running
 application instance.
 
@@ -395,6 +424,11 @@ X2GO_MIMEBOX_EXTENSIONS_BLACKLIST = [
 ]
 """Black-listed MIME box file extenstions."""
 
-# X2go desktop sharing
+# X2Go desktop sharing
 X2GO_SHARE_VIEWONLY=0
+"""Constant representing read-only access to shared desktops."""
 X2GO_SHARE_FULLACCESS=1
+"""Constant representing read-write (full) access to shared desktops."""
+
+PUBAPP_MAX_NO_SUBMENUS=10
+"""Less than ten applications will not get rendered into submenus."""
