@@ -1,41 +1,40 @@
 #!/usr/bin/env python
 
-# Copyright (C) 2010-2011 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2012 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
-# Python X2go is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
+# Python X2Go is free software; you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as published by
 # the Free Software Foundation; either version 3 of the License, or
 # (at your option) any later version.
 #
-# Python X2go is distributed in the hope that it will be useful,
+# Python X2Go is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# GNU Affero General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
+# You should have received a copy of the GNU Affero General Public License
 # along with this program; if not, write to the
 # Free Software Foundation, Inc.,
 # 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 
 """\
 Python Gevent based port forwarding server (openssh -L option) for the
-proxying of graphical X2go elements.
+proxying of graphical X2Go elements.
 
 """
 __NAME__ = "x2gofwtunnel-pylib"
 
 # modules
-import os, sys, copy
+import copy
 
 # gevent/greenlet
 import gevent
 from gevent import select, socket
 from gevent.server import StreamServer
 
-# Python X2go modules
+# Python X2Go modules
 import log
 from defaults import X2GOCLIENT_OS as _X2GOCLIENT_OS
-import x2go_exceptions
 
 class X2goFwServer(StreamServer):
     """\
@@ -50,15 +49,15 @@ class X2goFwServer(StreamServer):
         """\
         @param listener: listen on TCP/IP socket C{(<IP>, <Port>)}
         @type listener: C{tuple}
-        @param remote_host: hostname or IP of remote host (in case of X2go mostly 127.0.0.1)
+        @param remote_host: hostname or IP of remote host (in case of X2Go mostly 127.0.0.1)
         @type remote_host: C{str}
         @param remote_port: port of remote host
         @type remote_port: C{int}
         @param ssh_transport: a valid Paramiko/SSH transport object
-        @type ssh_transport: C{instance}
+        @type ssh_transport: C{obj}
         @param logger: you can pass an L{X2goLogger} object to the
             L{X2goFwServer} constructor
-        @type logger: C{instance}
+        @type logger: C{obj}
         @param loglevel: if no L{X2goLogger} object has been supplied a new one will be
             constructed with the given loglevel
         @type loglevel: C{int}
@@ -72,6 +71,7 @@ class X2goFwServer(StreamServer):
 
         self.chan = None
         self.is_active = False
+        self.failed = False
         self.keepalive = False
         self.chain_host = remote_host
         self.chain_port = remote_port
@@ -87,7 +87,7 @@ class X2goFwServer(StreamServer):
         Handle for SSH/Paramiko forwarding tunnel.
 
         @param fw_socket: local end of the forwarding tunnel
-        @type fw_socket: C{instance}
+        @type fw_socket: C{obj}
         @param address: unused/ignored
         @type address: C{tuple}
 
@@ -124,26 +124,24 @@ class X2goFwServer(StreamServer):
                                                                                           loglevel=log.loglevel_WARN)
                 gevent.sleep(.4)
 
-
         if not _success:
             self.logger('incoming request to %s:%d failed after %d attempts' % (self.chain_host,
                                                                                 self.chain_port,
                                                                                 _count),
                                                                                 loglevel=log.loglevel_ERROR)
+            if self.session_instance:
+                self.session_instance.HOOK_forwarding_tunnel_setup_failed(chain_host=self.chain_host, chain_port=self.chain_port)
+            self.failed = True
+
         else:
+
+            self.logger('connected!  Tunnel open %r -> %r -> %r' % (self.fw_socket.getpeername(),
+                        chan_peername, (self.chain_host, self.chain_port)),
+                        loglevel=log.loglevel_INFO)
+
             # once we are here, we can presume the tunnel to be active...
             self.is_active = True
 
-            if self.chan is None:
-                self.logger('incoming request to [%s]:%d was rejected by the SSH server.' %
-                            (self.chain_host, self.chain_port), loglevel=log.loglevel_ERROR)
-                if self.session_instance:
-                    self.session_instance.HOOK_forwarding_tunnel_setup_failed(chain_host=self.chain_host, chain_port=self.chain_port)
-                return
-            else:
-                self.logger('connected!  Tunnel open %r -> %r -> %r' % (self.fw_socket.getpeername(),
-                                                                chan_peername, (self.chain_host, self.chain_port)),
-                                                                loglevel=log.loglevel_INFO)
             self.keepalive = True
             try:
                 while self.keepalive:
@@ -222,7 +220,7 @@ def start_forward_tunnel(local_host='127.0.0.1', local_port=22022,
     """\
     Setup up a Paramiko/SSH port forwarding tunnel (like openssh -L option).
 
-    The tunnel is used to transport X2go graphics data through a proxy application like nxproxy.
+    The tunnel is used to transport X2Go graphics data through a proxy application like nxproxy.
 
     @param local_host: local starting point of the forwarding tunnel
     @type local_host: C{int}
@@ -232,40 +230,37 @@ def start_forward_tunnel(local_host='127.0.0.1', local_port=22022,
     @type remote_host: C{str}
     @param remote_port: ... on port C{<remote_port>}
     @type remote_port: C{int}
-    @param ssh_transport: the Paramiko/SSH transport (i.e. the X2go sessions Paramiko/SSH transport object)
-    @type ssh_transport: C{instance}
+    @param ssh_transport: the Paramiko/SSH transport (i.e. the X2Go sessions Paramiko/SSH transport object)
+    @type ssh_transport: C{obj}
     @param session_instance: the L{X2goSession} instance that initiates this tunnel
-    @type session_instance: C{instance}
+    @type session_instance: C{obj}
     @param logger: an X2goLogger object
-    @type logger: C{instance}
+    @type logger: C{obj}
 
     @return: returns an L{X2goFwServer} instance
-    @rtype: C{instance}
+    @rtype: C{obj}
 
     """
+    fw_server = X2goFwServer(listener=(local_host, local_port),
+                             remote_host=remote_host, remote_port=remote_port,
+                             ssh_transport=ssh_transport, session_instance=session_instance,
+                             logger=logger,
+                            )
     try:
-        fw_server = X2goFwServer(listener=(local_host, local_port),
-                                 remote_host=remote_host, remote_port=remote_port,
-                                 ssh_transport=ssh_transport, session_instance=session_instance,
-                                 logger=logger,
-                                )
-        try:
-            fw_server.start()
-            return fw_server
-        except socket.error:
-            pass
-    except x2go_exceptions.X2goFwTunnelException:
+        fw_server.start()
+    except socket.error:
+        fw_server.failed = True
+        fw_server.is_active = False
         pass
 
-    return None
-
+    return fw_server
 
 def stop_forward_tunnel(fw_server):
     """\
     Tear down a given Paramiko/SSH port forwarding tunnel.
 
     @param fw_server: an L{X2goFwServer} instance as returned by the L{start_forward_tunnel()} function
-    @type fw_server: C{instance}
+    @type fw_server: C{obj}
 
     """
     if fw_server is not None:
