@@ -1,18 +1,18 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2011 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2012 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
-# Python X2go is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
+# Python X2Go is free software; you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as published by
 # the Free Software Foundation; either version 3 of the License, or
 # (at your option) any later version.
 #
-# Python X2go is distributed in the hope that it will be useful,
+# Python X2Go is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# GNU Affero General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
+# You should have received a copy of the GNU Affero General Public License
 # along with this program; if not, write to the
 # Free Software Foundation, Inc.,
 # 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
@@ -36,7 +36,7 @@ import types
 import cStringIO
 import copy
 
-# Python X2go modules
+# Python X2Go modules
 from defaults import LOCAL_HOME as _current_home
 import log
 import utils
@@ -45,7 +45,7 @@ class X2goIniFile(object):
     """
     Base class for processing the different ini files used by X2go
     clients. Primarily used to standardize the content of the different
-    X2go client ini file (settings, printing, sessions, xconfig).
+    X2Go client ini file (settings, printing, sessions, xconfig).
 
     If entries are omitted in an ini file, they are filled with
     default values (as hard coded in Python X2go), so the resulting objects 
@@ -67,13 +67,13 @@ class X2goIniFile(object):
         @type config_files: C{list}
         @param defaults: a cascaded Python dicitionary structure with ini file defaults (to override
             Python X2go's hard coded defaults in L{defaults}
-        @type defaults: dict
+        @type defaults: C{dict}
         @param logger: you can pass an L{X2goLogger} object to the
-            L{X2goFwServer} constructor
+            L{X2goIniFile} constructor
         @type logger: L{X2goLogger} instance
         @param loglevel: if no L{X2goLogger} object has been supplied a new one will be
             constructed with the given loglevel
-        @type loglevel: int
+        @type loglevel: C{int}
 
         """
         # make sure a None type gets turned into list type
@@ -93,9 +93,9 @@ class X2goIniFile(object):
 
         # we purposefully do not inherit the SafeConfigParser class
         # here as we do not want to run into name conflicts between
-        # X2go ini file options and method / property names in
+        # X2Go ini file options and method / property names in
         # SafeConfigParser... This is a pre-cautious approach...
-        self.iniConfig = ConfigParser.SafeConfigParser()
+        self.iniConfig = ConfigParser.SafeConfigParser(self.defaultValues)
         self.iniConfig.optionxform = str
 
         _create_file = False
@@ -122,7 +122,7 @@ class X2goIniFile(object):
         self.logger('config files found: %s' % _found_config_files or 'none', loglevel=log.loglevel_INFO, )
 
         for file_name in _found_config_files:
-            if file_name.startswith(_current_home):
+            if file_name.startswith(os.path.normpath(_current_home)):
                 # we will use the first file found in the user's home dir for writing modifications
                 self.user_config_file = file_name
                 break
@@ -166,7 +166,7 @@ class X2goIniFile(object):
     def _fill_defaults(self):
         """\
         Fills a C{SafeConfigParser} object with the default ini file 
-        values as pre-defined in Python X2go or. This SafeConfigParser 
+        values as pre-defined in Python X2Go or. This SafeConfigParser 
         object is held in RAM. No configuration file is affected by this 
         method.
 
@@ -247,13 +247,15 @@ class X2goIniFile(object):
             elif key_type is types.IntType:
                 return self.iniConfig.getint(section, key)
             elif key_type is types.ListType:
-                val = self.iniConfig.get(section, key)
-                if val.startswith('[') and val.endswith(']'):
-                    return eval(val)
-                elif ',' in val:
-                    val = [ v.strip() for v in val.split(',') ]
+                _val = self.iniConfig.get(section, key)
+                _val = _val.strip()
+                if _val.startswith('[') and _val.endswith(']'):
+                    return eval(_val)
+                elif ',' in _val:
+                    _val = [ v.strip() for v in _val.split(',') ]
                 else:
-                    val = [ val ]
+                    _val = [ _val ]
+                return _val
             else:
                 _val = self.iniConfig.get(section, key)
                 return _val.decode(utils.get_encoding())
