@@ -1,47 +1,46 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2011 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2012 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 
-# The Python X2go sFTPServer code was originally written by Richard Murri, 
+# The Python X2Go sFTPServer code was originally written by Richard Murri, 
 # for further information see his website: http://www.richardmurri.com
 
-# Python X2go is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
+# Python X2Go is free software; you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as published by
 # the Free Software Foundation; either version 3 of the License, or
 # (at your option) any later version.
 #
-# Python X2go is distributed in the hope that it will be useful,
+# Python X2Go is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# GNU Affero General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
+# You should have received a copy of the GNU Affero General Public License
 # along with this program; if not, write to the
 # Free Software Foundation, Inc.,
 # 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 
 """\
-For sharing local folders via sFTP/sshfs Python X2go implements its own sFTP 
+For sharing local folders via sFTP/sshfs Python X2Go implements its own sFTP 
 server (as end point of reverse forwarding tunnel requests). Thus, Python X2go
 does not need a locally installed SSH daemon on the client side machine.
 
-The Python X2go sFTP server code was originally written by Richard Murri, 
+The Python X2Go sFTP server code was originally written by Richard Murri, 
 for further information see his website: http://www.richardmurri.com
 
 """
 __NAME__ = "x2gosftpserver-pylib"
 
-import base64, os, sys
+import os
 import shutil
-import copy, types
+import copy
 import threading
 import paramiko
 import gevent
 
-# Python X2go modules
+# Python X2Go modules
 import rforward
 import defaults
-import utils
 import log
 
 class _SSHServer(paramiko.ServerInterface):
@@ -59,7 +58,7 @@ class _SSHServer(paramiko.ServerInterface):
         @param session_instance: the calling L{X2goSession} instance
         @type session_instance: L{X2goSession} instance
         @param logger: you can pass an L{X2goLogger} object to the L{X2goClientXConfig} constructor
-        @type logger: C{instance}
+        @type logger: C{obj}
         @param loglevel: if no L{X2goLogger} object has been supplied a new one will be
             constructed with the given loglevel
         @type loglevel: C{int}
@@ -85,6 +84,7 @@ class _SSHServer(paramiko.ServerInterface):
         @type kind: C{str}
         @param chanid: channel id (unused)
         @type chanid: C{any}
+
         @return: returns a Paramiko/SSH return code
         @rtype: C{int}
 
@@ -102,8 +102,10 @@ class _SSHServer(paramiko.ServerInterface):
         @type username: C{str}
         @param key: incoming SSH key to be used for authentication
         @type key: C{paramiko.RSAKey} instance
+
         @return: returns a Paramiko/SSH return code
         @rtype: C{int}
+
         """
         if username == self.current_local_user:
             self.logger('sFTP server %s: username is %s' % (self, self.current_local_user), loglevel=log.loglevel_DEBUG)
@@ -119,6 +121,7 @@ class _SSHServer(paramiko.ServerInterface):
 
         @param username: username of incoming authentication request
         @type username: C{str}
+
         @return: statically returns C{publickey} as auth mechanism
         @rtype: C{str}
 
@@ -133,6 +136,13 @@ class _SFTPHandle(paramiko.SFTPHandle):
 
     """
     def stat(self):
+        """\
+        Create an SFTPAttributes object from an existing stat object (an object returned by os.stat).
+
+        return: new C{SFTPAttributes} object with the same attribute fields.
+        rtype: C{obj}
+
+        """
         try:
             return paramiko.SFTPAttributes.from_stat(os.fstat(self.readfile.fileno()))
         except OSError, e:
@@ -153,7 +163,7 @@ class _SFTPServerInterface(paramiko.SFTPServerInterface):
         @param chroot: chroot environment for this SFTP interface
         @type chroot: C{str}
         @param logger: you can pass an L{X2goLogger} object to the L{X2goClientXConfig} constructor
-        @type logger: C{instance}
+        @type logger: C{obj}
         @param loglevel: if no L{X2goLogger} object has been supplied a new one will be
             constructed with the given loglevel
         @type loglevel: C{int}
@@ -178,8 +188,10 @@ class _SFTPServerInterface(paramiko.SFTPServerInterface):
 
         @param path: path name within chroot
         @type path: C{str}
+
         @return: real path name (including drive letter on Windows systems)
         @rtype: C{str}
+
         """
         if defaults.X2GOCLIENT_OS == 'Windows' and path.startswith('/windrive'):
             _path_components = path.split('/')
@@ -231,6 +243,7 @@ class _SFTPServerInterface(paramiko.SFTPServerInterface):
 
         @param path: path to file/folder
         @type path: C{str}
+
         @return: returns the file's stat output, on failure: returns a Paramiko/SSH return code
         @rtype: C{class} or C{int}
 
@@ -249,6 +262,7 @@ class _SFTPServerInterface(paramiko.SFTPServerInterface):
 
         @param path: path to folder
         @type path: C{str}
+
         @return: returns the file's lstat output, on failure: returns a Paramiko/SSH return code
         @rtype: C{class} or C{int}
 
@@ -271,6 +285,7 @@ class _SFTPServerInterface(paramiko.SFTPServerInterface):
         @type flags: C{str}
         @param attr: file attributes
         @type attr: C{class}
+
         @return: file handle/object for remote file, on failure: returns a Paramiko/SSH return code
         @rtype: L{_SFTPHandle} instance or C{int}
 
@@ -323,8 +338,10 @@ class _SFTPServerInterface(paramiko.SFTPServerInterface):
 
         @param path: path to file
         @type path: C{str}
+
         @return: returns Paramiko/SSH return code
         @rtype: C{int}
+
         """
         path = self._realpath(path)
         os.remove(path)
@@ -333,12 +350,13 @@ class _SFTPServerInterface(paramiko.SFTPServerInterface):
 
     def rename(self, oldpath, newpath):
         """\
-        Rename/Move a file.
+        Rename/move a file.
 
         @param oldpath: old path/location/file name
         @type oldpath: C{str}
         @param newpath: new path/location/file name
         @type newpath: C{str}
+
         @return: returns Paramiko/SSH return code
         @rtype: C{int}
 
@@ -361,6 +379,7 @@ class _SFTPServerInterface(paramiko.SFTPServerInterface):
         @type path: C{str}
         @param attr: file attributes
         @type attr: C{class}
+
         @return: returns Paramiko/SSH return code
         @rtype: C{int}
 
@@ -380,6 +399,7 @@ class _SFTPServerInterface(paramiko.SFTPServerInterface):
 
         @param path: folder to be removed
         @type path: C{str}
+
         @return: returns Paramiko/SSH return code
         @rtype: C{int}
 
@@ -401,6 +421,7 @@ class _SFTPServerInterface(paramiko.SFTPServerInterface):
         @type path: C{str}
         @param attr: new file attributes
         @type attr: C{class}
+
         @return: returns Paramiko/SSH return code
         @rtype: C{int}
 
@@ -425,6 +446,7 @@ class _SFTPServerInterface(paramiko.SFTPServerInterface):
         @type target_path: C{str}
         @param path: link location
         @type path: C{str}
+
         @return: returns Paramiko/SSH return code
         @rtype: C{int}
 
@@ -446,8 +468,10 @@ class _SFTPServerInterface(paramiko.SFTPServerInterface):
 
         @param path: path of symbolic link
         @type path: C{str}
+
         @return: target location of the symbolic link, on failure: returns a Paramiko/SSH return code
         @rtype: C{str} or C{int}
+
         """
         path = self._realpath(path)
         try:
@@ -469,8 +493,8 @@ class _SFTPServerInterface(paramiko.SFTPServerInterface):
 class X2goRevFwTunnelToSFTP(rforward.X2goRevFwTunnel):
     """\
     A reverse fowarding tunnel with an sFTP server at its endpoint. This blend of a Paramiko/SSH
-    reverse forwarding tunnel is used to provide access to local X2go client folders
-    from within the the remote X2go server session.
+    reverse forwarding tunnel is used to provide access to local X2Go client folders
+    from within the the remote X2Go server session.
 
     """
     def __init__(self, server_port, ssh_transport, auth_key=None, session_instance=None, logger=None, loglevel=log.loglevel_DEFAULT):
@@ -478,9 +502,9 @@ class X2goRevFwTunnelToSFTP(rforward.X2goRevFwTunnel):
         Start a Paramiko/SSH reverse forwarding tunnel, that has an sFTP server listening at
         the endpoint of the tunnel.
 
-        @param server_port: the TCP/IP port on the X2go server (starting point of the tunnel), 
+        @param server_port: the TCP/IP port on the X2Go server (starting point of the tunnel), 
             normally some number above 30000
-        @type server_port: int
+        @type server_port: C{int}
         @param ssh_transport: the L{X2goSession}'s Paramiko/SSH transport instance
         @type ssh_transport: C{paramiko.Transport} instance
         @param auth_key: Paramiko/SSH RSAkey object that has to be authenticated against by
@@ -491,9 +515,10 @@ class X2goRevFwTunnelToSFTP(rforward.X2goRevFwTunnel):
         @type logger: L{X2goLogger} instance
         @param loglevel: if no L{X2goLogger} object has been supplied a new one will be
             constructed with the given loglevel
-        @type loglevel: int
+        @type loglevel: C{int}
 
         """
+        self.ready = False
         if logger is None:
             self.logger = log.X2goLogger(loglevel=loglevel)
         else:
@@ -522,28 +547,29 @@ class X2goRevFwTunnelToSFTP(rforward.X2goRevFwTunnel):
 
         L{X2goRevFwTunnelToSFTP.run()} waits for notifications of an appropriate incoming
         Paramiko/SSH channel (issued by L{X2goRevFwTunnelToSFTP.notify()}). Appropriate in
-        this context means, that its starting point on the X2go server matches the class's
+        this context means, that its starting point on the X2Go server matches the class's
         property C{server_port}.
 
         Once a new incoming channel gets announced by the L{notify()} method, a new 
         L{X2goRevFwSFTPChannelThread} instance will be initialized. As a data stream handler,
         the function L{x2go_rev_forward_sftpchannel_handler()} will be used.
 
-        The channel will last till the connection gets dropped on the X2go server side or 
+        The channel will last till the connection gets dropped on the X2Go server side or 
         until the tunnel gets paused by an L{X2goRevFwTunnelToSFTP.pause()} call or 
         stopped via the C{X2goRevFwTunnelToSFTP.stop_thread()} method.
 
         """
         self._request_port_forwarding()
         self._keepalive = True
+        self.ready = True
         while self._keepalive:
 
             self.incoming_channel.acquire()
 
-            self.logger('waiting for incoming sFTP channel on X2go server port: [localhost]:%s' % self.server_port, loglevel=log.loglevel_DEBUG)
+            self.logger('waiting for incoming sFTP channel on X2Go server port: [localhost]:%s' % self.server_port, loglevel=log.loglevel_DEBUG)
             self.incoming_channel.wait()
             if self._keepalive:
-                self.logger('Detected incoming sFTP channel on X2go server port: [localhost]:%s' % self.server_port, loglevel=log.loglevel_DEBUG)
+                self.logger('Detected incoming sFTP channel on X2Go server port: [localhost]:%s' % self.server_port, loglevel=log.loglevel_DEBUG)
                 _chan = self.ssh_transport.accept()
                 self.logger('sFTP channel %s for server port [localhost]:%s is up' % (_chan, self.server_port), loglevel=log.loglevel_DEBUG)
             else:
@@ -561,6 +587,7 @@ class X2goRevFwTunnelToSFTP(rforward.X2goRevFwTunnel):
                                                              )
                 _new_chan_thread.start()
                 self.open_channels['[%s]:%s' % _chan.origin_addr] = _new_chan_thread
+        self.ready = False
 
 
 def x2go_rev_forward_sftpchannel_handler(chan=None, auth_key=None, logger=None):
@@ -574,7 +601,7 @@ def x2go_rev_forward_sftpchannel_handler(chan=None, auth_key=None, logger=None):
         - ... if the L{X2goRevFwTunnelToSFTP} parent thread gets paused. The call
         of L{X2goRevFwTunnelToSFTP.pause()} on the instance can be used to shut down all incoming 
         tunneled SSH connections associated to this L{X2goRevFwTunnelToSFTP} instance
-        from within a Python X2go application.
+        from within a Python X2Go application.
 
     @param chan: an incoming sFTP channel
     @type chan: paramiko.Channel instance
