@@ -1,18 +1,18 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2011 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2012 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
-# Python X2go is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
+# Python X2Go is free software; you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as published by
 # the Free Software Foundation; either version 3 of the License, or
 # (at your option) any later version.
 #
-# Python X2go is distributed in the hope that it will be useful,
+# Python X2Go is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# GNU Affero General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
+# You should have received a copy of the GNU Affero General Public License
 # along with this program; if not, write to the
 # Free Software Foundation, Inc.,
 # 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
@@ -29,17 +29,16 @@ import os
 import copy
 import paramiko
 import threading
-import socket
 
 import string
 import random
 
-# Python X2go modules
-import x2go.forward as forward
-import x2go.checkhosts as checkhosts
-import x2go.log as log
-import x2go.utils as utils
-from x2go.x2go_exceptions import *
+# Python X2Go modules
+import forward
+import checkhosts
+import log
+import utils
+import x2go_exceptions
 
 from x2go.defaults import CURRENT_LOCAL_USER as _CURRENT_LOCAL_USER
 from x2go.defaults import LOCAL_HOME as _LOCAL_HOME
@@ -50,7 +49,7 @@ monkey_patch_paramiko()
 
 class X2goSSHProxy(paramiko.SSHClient, threading.Thread):
     """\
-    X2goSSHProxy can be used to proxy X2go connections through a firewall via SSH.
+    X2goSSHProxy can be used to proxy X2Go connections through a firewall via SSH.
 
     """
     fw_tunnel = None
@@ -65,13 +64,14 @@ class X2goSSHProxy(paramiko.SSHClient, threading.Thread):
                  session_instance=None,
                  logger=None, loglevel=log.loglevel_DEFAULT, ):
         """\
-        Initialize an X2goSSHProxy instance. Use an instance of this class to tunnel X2go requests through
+        Initialize an X2goSSHProxy instance. Use an instance of this class to tunnel X2Go requests through
         a proxying SSH server (i.e. to subLANs that are separated by firewalls or to private IP subLANs that
         are NATted behind routers).
 
         @param username: login user name to be used on the SSH proxy host
         @type username: C{str}
-        @param password: user's password on the SSH proxy host
+        @param password: user's password on the SSH proxy host, with private key authentication it will be
+            used to unlock the key (if needed)
         @type password: C{str}
         @param key_filename: name of a SSH private key file
         @type key_filename: C{str}
@@ -121,6 +121,8 @@ class X2goSSHProxy(paramiko.SSHClient, threading.Thread):
             constructed with the given loglevel
         @type loglevel: int
 
+        @raise X2goSSHProxyAuthenticationException: if the SSH proxy caused a C{paramiko.AuthenticationException}
+        @raise X2goSSHProxyException: if the SSH proxy caused a C{paramiko.SSHException}
         """
         if logger is None:
             self.logger = log.X2goLogger(loglevel=loglevel)
@@ -171,6 +173,11 @@ class X2goSSHProxy(paramiko.SSHClient, threading.Thread):
             username = _CURRENT_LOCAL_USER
 
         self._keepalive = True
+        self.session_instance = session_instance
+
+        self.client_instance = None
+        if self.session_instance is not None:
+            self.client_instance = self.session_instance.get_client_instance()
 
         self.ssh_rootdir = ssh_rootdir
         paramiko.SSHClient.__init__(self)
@@ -189,20 +196,36 @@ class X2goSSHProxy(paramiko.SSHClient, threading.Thread):
         try:
             if (key_filename and os.path.exists(os.path.normpath(key_filename))) or pkey:
                 try:
-                    self.connect(_hostname, port=self.port, 
-                                 username=self.username, 
-                                 key_filename=key_filename,
-                                 pkey=pkey,
-                                 look_for_keys=False,
-                                 allow_agent=False,
-                                )
-                except AuthenticationException, e:
+                    if password:
+                        self.connect(_hostname, port=self.port, 
+                                     username=self.username,
+                                     password=password,
+                                     key_filename=key_filename,
+                                     pkey=pkey,
+                                     look_for_keys=False,
+                                     allow_agent=False,
+                                    )
+                    else:
+                        self.connect(_hostname, port=self.port, 
+                                     username=self.username, 
+                                     key_filename=key_filename,
+                                     pkey=pkey,
+                                     look_for_keys=False,
+                                     allow_agent=False,
+                                    )
+
+                except x2go_exceptions.AuthenticationException, e:
                     self.close()
-                    raise X2goSSHProxyAuthenticationException('pubkey auth mechanisms both failed')
+                    raise x2go_exceptions.X2goSSHProxyAuthenticationException('pubkey auth mechanisms both failed')
                 except:
                     self.close()
                     raise
 
+                # since Paramiko 1.7.7.1 there is compression available, let's use it if present...
+                t = self.get_transport()
+                if hasattr(t, 'use_compression'):
+                    t.use_compression(compress=True)
+
             # if there is not private key, we will use the given password, if any
             else:
                 # create a random password if password is empty to trigger host key validity check
@@ -215,16 +238,16 @@ class X2goSSHProxy(paramiko.SSHClient, threading.Thread):
                                  look_for_keys=False,
                                  allow_agent=False,
                                 )
-                except AuthenticationException:
+                except x2go_exceptions.AuthenticationException:
                     self.close()
-                    raise X2goSSHProxyAuthenticationException('interactive auth mechanisms failed')
+                    raise x2go_exceptions.X2goSSHProxyAuthenticationException('interactive auth mechanisms failed')
                 except:
                     self.close()
                     raise
 
-        except paramiko.SSHException, e:
+        except x2go_exceptions.SSHException, e:
             self.close()
-            raise X2goSSHProxyException(str(e))
+            raise x2go_exceptions.X2goSSHProxyException(str(e))
         except:
             self.close()
             raise
@@ -238,7 +261,6 @@ class X2goSSHProxy(paramiko.SSHClient, threading.Thread):
         Wraps around a Paramiko/SSH host key check.
 
         """
-
         # hostname rewrite for localhost, force to IPv4
         _hostname = self.hostname
 
@@ -256,9 +278,20 @@ class X2goSSHProxy(paramiko.SSHClient, threading.Thread):
         """\
         Start the SSH proxying tunnel...
 
+        @raise X2goSSHProxyException: if the SSH proxy could not retrieve an SSH transport for proxying a X2Go server-client connection
+
         """
         if self.get_transport() is not None and self.get_transport().is_authenticated():
             self.local_port = utils.detect_unused_port(bind_address=self.local_host, preferred_port=self.local_port)
+            if self.client_instance is not None:
+                _profile_id = self.session_instance.get_profile_id()
+                if self.client_instance.session_profiles.has_profile(_profile_id):
+                    self.client_instance.session_profiles.update_value(_profile_id,
+                                                                       'sshproxytunnel',
+                                                                       '%s:%s:%s:%s' % (self.local_host, self.local_port, self.remote_host, self.remote_port)
+                                                                      )
+                self.client_instance.session_profiles.write_user_config = True
+                self.client_instance.session_profiles.write()
             self.fw_tunnel = forward.start_forward_tunnel(local_host=self.local_host,
                                                           local_port=self.local_port,
                                                           remote_host=self.remote_host,
@@ -272,7 +305,7 @@ class X2goSSHProxy(paramiko.SSHClient, threading.Thread):
                 gevent.sleep(.1)
 
         else:
-            raise X2goSSHProxyException('SSH proxy connection could not retrieve an SSH transport')
+            raise x2go_exceptions.X2goSSHProxyException('SSH proxy connection could not retrieve an SSH transport')
 
     def get_local_proxy_port(self):
         """\
@@ -280,6 +313,7 @@ class X2goSSHProxy(paramiko.SSHClient, threading.Thread):
 
         @return: local IP socket port
         @rtype: C{int}
+
         """
         return self.local_port
 
